from logging import getLogger
from os import name
from platform import architecture, uname, python_implementation, python_version
from sys import platform

from moht.log import config_logger

LOG = getLogger(__name__)
config_logger(logger=LOG, verbose=False)
LOG.debug(f'Arch: {name} / {platform} / {" / ".join(architecture())}')
LOG.debug(f'Python: {python_implementation()}-{python_version()}')
LOG.debug(f'{uname()}')

VERSION = '0.1.0'
PLUGINS2CLEAN = [
    "Abandoned_Flatv2_0.esp", "Almalexia_Voicev1.esp", "FLG - Balmora's Underworld V1.1.esp", "BitterAndBlighted.ESP", "Building Up Uvirith's Legacy1.1.ESP",
    "Caldera.esp", "DD_Caldera_Expansion.esp", "NX9_Guards_Complete.ESP", "Radiant Gem.esp", "Dwemer and Ebony Service Refusal.ESP",
    "Foyada Mamaea Overhaul.ESP", "Graphic Herbalism.esp", "Graphic Herbalism - No Glow.esp", "Graphic Herbalism Extra.esp", "Hla Oad.esp",
    "CultSheog-1.02.ESP", "CultSheog-TR1807.esp", "Kilcunda's Balmora.ESP", "MD_Azurian Isles.esm", "Magical Missions.ESP", "Mannequins for Sale.esp",
    "Xenn's Marksman Overhaul.ESP", "Meteorite Ministry Palace - Higher.ESP", "MW Containers Animated.esp", "Go To Jail.esp",
    "Go To Jail (Mournhold + Solshteim).ESP", "MRM.esm", "NX9_Guards_Complete.ESP", "OAAB - The Ashen Divide.ESP", "On the Move.esp",
    "Ports Of Vvardenfell V1.6.ESP", "Quill of Feyfolken 2.0.esp", "Library of Vivec Overhaul - Full.esp", "SadrithMoraExpandedTR.esp", "Sanctus Shrine.esp",
    "DA_Sobitur_Facility_Clean.ESP", "DA_Sobitur_Quest_Part_1 Clean.esp", "DA_Sobitur_Quest_Part_2 Clean.esp", "DA_Sobitur_Repurposed_1.ESP",
    "DA_Sobitur_TRIngred_Compat.ESP", "Stav_gnisis_minaret.ESP", "OTR_Coast_Variety.esp", "TheForgottenShields - Artifacts_NG.esp", "SG-toughersixth.esp",
    "Ttooth's Missing NPCs - No Nolus.ESP", "True_Lights_And_Darkness_1.1.esp", "UCNature.esm", "UFR_v3dot2_noRobe.esp", "Vurt's BC Tree Replacer II.ESP",
    "Windows Glow - Bloodmoon Eng.esp", "Windows Glow - Raven Rock Eng.esp", "Windows Glow - Tribunal Eng.esp", "Windows Glow.esp",
]



import tkinter as tk
from os import path

from moht.tk_gui import MohtTkGui


def run():
    """Function to start Mod Helper Tool GUI."""
    root = tk.Tk()
    width, height = 500, 200
    root.geometry(f'{width}x{height}')
    root.minsize(width=width, height=height)
    here = path.abspath(path.dirname(__file__))
    root.iconphoto(False, tk.PhotoImage(file=path.join(here, 'moht.png')))
    gui = MohtTkGui(master=root)
    gui.mainloop()


if __name__ == '__main__':
    run()



from datetime import datetime
from logging import DEBUG, StreamHandler, INFO, Formatter, Logger
from logging.handlers import RotatingFileHandler
from os import path
from tempfile import gettempdir


def config_logger(logger: Logger, verbose=False) -> None:
    """
    Configure global logger add handlers and set formatters.

    :param logger:
    :param verbose: turn on/off verbose mode
    """
    logger.setLevel(DEBUG)
    file_hand = RotatingFileHandler(filename=path.join(gettempdir(), 'moht.log'), mode='a', encoding='utf-8', maxBytes=5 * 1024 * 1024, backupCount=1)
    file_hand.setLevel(DEBUG)
    file_hand.setFormatter(Formatter('%(asctime)s | %(name)-17s | %(levelname)-7s | %(threadName)-10s | %(message)s / %(funcName)s:%(lineno)d'))
    stream_hand = StreamHandler()
    stream_hand.setLevel(INFO)
    if verbose:
        stream_hand.setLevel(DEBUG)
    stream_hand.setFormatter(Formatter('%(levelname)-7s | %(message)s'))
    logger.addHandler(stream_hand)
    logger.addHandler(file_hand)
    header = '#' * 60
    logger.debug(f'\n{header}\nStart session: {datetime.now()}\n{header}')
    logger.info(f'Log file store at: {file_hand.baseFilename}')



import tkinter as tk
from functools import partial
from logging import getLogger
from os import path, removedirs, chdir, walk, remove, sep
from pathlib import Path
from pprint import pformat
from shlex import split
from shutil import move, copy2, rmtree
from subprocess import Popen, PIPE
from sys import platform
from time import time
from tkinter import filedialog, messagebox

from moht import PLUGINS2CLEAN, VERSION
from moht.utils import parse_cleaning

LOG = getLogger(__name__)


class MohtTkGui(tk.Frame):
    def __init__(self, master: tk.Tk,) -> None:
        """
        Create basic GUI for Mod Helper Tool application.

        :param master: Top level widget
        """
        LOG.info(f'moht v{VERSION} https://gitlab.com/modding-openmw/modhelpertool')
        super().__init__(master)
        self.tes3cmd = 'tes3cmd-0.37v.exe' if platform == 'win32' else 'tes3cmd-0.37w'
        self.master = master
        self.master.title('Mod Helper Tool')
        self.statusbar = tk.StringVar()
        self._mods_dir = tk.StringVar()
        self._morrowind_dir = tk.StringVar()
        self.chkbox_backup = tk.BooleanVar()
        self.chkbox_cache = tk.BooleanVar()
        self.stats = {'all': 0, 'cleaned': 0, 'clean': 0, 'error': 0}
        self._init_widgets()
        self.statusbar.set(f'ver. {VERSION}')
        # self._mods_dir.set('/home/emc/.local/share/openmw/data')
        # self._mods_dir.set('/home/emc/CitiesTowns/')  # test linux
        # self._mods_dir.set('D:/CitiesTowns')  # test win
        self._mods_dir.set(str(Path.home()))
        # self._morrowind_dir.set('/home/emc/.wine/drive_c/Morrowind/Data Files/')  # test linux
        # self._morrowind_dir.set('S:/Program Files/Morrowind/Data Files')  # test win
        self._morrowind_dir.set(str(Path.home()))
        self.chkbox_backup.set(True)
        self.chkbox_cache.set(True)
        self._check_clean_bin()

    def _init_widgets(self) -> None:
        self.master.columnconfigure(index=0, weight=10)
        self.master.columnconfigure(index=1, weight=1)
        self.master.rowconfigure(index=0, weight=10)
        self.master.rowconfigure(index=1, weight=1)
        self.master.rowconfigure(index=2, weight=1)
        self.master.rowconfigure(index=3, weight=1)
        self.master.rowconfigure(index=4, weight=1)

        mods_dir = tk.Entry(master=self.master, textvariable=self._mods_dir)
        morrowind_dir = tk.Entry(master=self.master, textvariable=self._morrowind_dir)
        mods_btn = tk.Button(master=self.master, text='Select Mods Dir', width=16, command=partial(self.select_dir, self._mods_dir))
        morrowind_btn = tk.Button(master=self.master, text='Select Morrowind Dir', width=16, command=partial(self.select_dir, self._morrowind_dir))
        self.clean_btn = tk.Button(master=self.master, text='Clean Mods', width=16, command=self.start_clean)
        self.report_btn = tk.Button(master=self.master, text='Report', width=16, state=tk.DISABLED, command=self.report)
        close_btn = tk.Button(master=self.master, text='Close Tool', width=16, command=self.master.destroy)
        statusbar = tk.Label(master=self.master, textvariable=self.statusbar)
        chkbox_backup = tk.Checkbutton(master=self.master, text='Remove backup after successful clean-up', variable=self.chkbox_backup)
        chkbox_cache = tk.Checkbutton(master=self.master, text='Remove cache after successful clean-up', variable=self.chkbox_cache)

        mods_dir.grid(row=0, column=0, padx=2, pady=2, sticky=f'{tk.W}{tk.E}')
        morrowind_dir.grid(row=1, column=0, padx=2, pady=2, sticky=f'{tk.W}{tk.E}')
        chkbox_backup.grid(row=2, column=0, padx=2, pady=2, sticky=tk.W)
        chkbox_cache.grid(row=3, column=0, padx=2, pady=2, sticky=tk.W)
        mods_btn.grid(row=0, column=1, padx=2, pady=2)
        morrowind_btn.grid(row=1, column=1, padx=2, pady=2)
        self.clean_btn.grid(row=2, column=1, padx=2, pady=2)
        self.report_btn.grid(row=3, column=1, padx=2, pady=2)
        close_btn.grid(row=4, column=1, padx=2, pady=2)
        statusbar.grid(row=5, column=0, columnspan=3, sticky=tk.W)

    @staticmethod
    def select_dir(text_var: tk.StringVar) -> None:
        """
        Select directory location.

        :param text_var: StringVar of Entry to update
        """
        directory = filedialog.askdirectory(initialdir=str(Path.home()), title='Select directory')
        LOG.debug(f'Directory: {directory}')
        text_var.set(f'{directory}')

    def start_clean(self) -> None:
        """Start cleaning process."""
        all_plugins = [Path(path.join(root, filename)) for root, _, files in walk(self.mods_dir) for filename in files if filename.lower().endswith('.esp') or filename.lower().endswith('.esm')]
        LOG.debug(f'all_plugins: {len(all_plugins)}: {all_plugins}')
        plugins_to_clean = [plugin_file for plugin_file in all_plugins if str(plugin_file).split(sep)[-1] in PLUGINS2CLEAN]
        no_of_plugins = len(plugins_to_clean)
        LOG.debug(f'to_clean: {no_of_plugins}: {plugins_to_clean}')
        chdir(self.morrowind_dir)
        here = path.abspath(path.dirname(__file__))
        self.stats = {'all': no_of_plugins, 'cleaned': 0, 'clean': 0, 'error': 0}
        start = time()
        for idx, plug in enumerate(plugins_to_clean, 1):
            LOG.debug(f'---------------------------- {idx} / {no_of_plugins} ---------------------------- ')
            LOG.debug(f'Copy: {plug} -> {self.morrowind_dir}')
            copy2(plug, self.morrowind_dir)
            mod_file = str(plug).split(sep)[-1]
            cmd_str = f'{path.join(here, self.tes3cmd)} clean --output-dir --overwrite "{mod_file}"'
            cmd = split(cmd_str) if platform == 'linux' else cmd_str
            LOG.debug(f'CMD: {cmd}')
            stdout, stderr = Popen(cmd, stdout=PIPE, stderr=PIPE).communicate()
            out, err = stdout.decode('utf-8'), stderr.decode('utf-8')
            LOG.debug(f'Out: {out}')
            LOG.debug(f'Err: {err}')
            result, reason = parse_cleaning(out, err, mod_file)
            LOG.debug(f'Result: {result}, Reason: {reason}')
            self._update_stats(mod_file, plug, reason, result)
            if self.chkbox_backup.get():
                LOG.debug(f'Remove: {self.morrowind_dir}/{mod_file}')
                remove(f'{self.morrowind_dir}/{mod_file}')
        LOG.debug(f'---------------------------- Done: {no_of_plugins} ---------------------------- ')
        if self.chkbox_cache.get():
            removedirs(f'{self.morrowind_dir}/1')
            cachedir = 'tes3cmd' if platform == 'win32' else '.tes3cmd-3'
            rmtree(f'{self.morrowind_dir}/{cachedir}', ignore_errors=True)
        LOG.debug(f'Total time: {time() - start:.2f} s')
        self.statusbar.set('Done. See report!')
        self.report_btn.config(state=tk.NORMAL)

    def _update_stats(self, mod_file: str, plug: Path, reason: str, result: bool) -> None:
        if result:
            LOG.debug(f'Move: {self.morrowind_dir}/1/{mod_file} -> {plug}')
            move(f'{self.morrowind_dir}/1/{mod_file}', plug)
            self.stats['cleaned'] += 1
        if not result and reason == 'not modified':
            self.stats['clean'] += 1
        if not result and 'not found' in reason:
            self.stats['error'] += 1
            esm = self.stats.get(reason, 0)
            esm += 1
            self.stats.update({reason: esm})

    def report(self) -> None:
        """Show report after clean-up."""
        LOG.debug(f'Report: {self.stats}')
        messagebox.showinfo('Report', pformat(self.stats, width=15))
        self.report_btn.config(state=tk.DISABLED)
        self.statusbar.set(f'ver. {VERSION}')

    def _check_clean_bin(self):
        here = path.abspath(path.dirname(__file__))
        LOG.debug('Checking tes3cmd')
        cmd_str = f'{path.join(here, self.tes3cmd)} -h'
        cmd = split(cmd_str) if platform == 'linux' else cmd_str
        LOG.debug(f'CMD: {cmd}')
        stdout, stderr = Popen(cmd, stdout=PIPE, stderr=PIPE).communicate()
        out, err = stdout.decode('utf-8'), stderr.decode('utf-8')
        result, reason = parse_cleaning(out, err, '')
        LOG.debug(f'Result: {result}, Reason: {reason}')
        if not result and 'Config::IniFiles' in reason:
            msg = 'Use your package manager, check for `perl-Config-IniFiles` or a similar package.\n\nOr run from a terminal:\ncpan install Config::IniFiles'
            messagebox.showerror('Missing package', msg)
            self.statusbar.set(f'Error: {reason}')
            self.clean_btn.config(state=tk.DISABLED)

    @property
    def mods_dir(self) -> str:
        """
        Get root of mods directory.

        :return: mods dir as string
        """
        return str(self._mods_dir.get())

    @mods_dir.setter
    def mods_dir(self, value: Path) -> None:
        self._mods_dir.set(str(value))

    @property
    def morrowind_dir(self) -> str:
        """
        Get Morrowind Data Files directory.

        :return: morrowind as sring
        """
        return str(self._morrowind_dir.get())

    @morrowind_dir.setter
    def morrowind_dir(self, value: Path) -> None:
        self._morrowind_dir.set(str(value))



from os import linesep
from re import search, MULTILINE
from typing import Tuple


def parse_cleaning(out: str, err: str, mod_filename: str) -> Tuple[bool, str]:  # type: ignore
    """
    Parse output of cleaning command printout.

    :param out: Command STANDARD OUTPUT
    :param err: Command STANDARD ERROR
    :param mod_filename: Mod filename
    :return: Result and reason
    """
    ceases = {
        1: {'args': (r'\[ERROR \({}\): Master: (.* not found) in <DATADIR>]'.format(mod_filename), err, MULTILINE),
            'result': False},
        2: {'args': (r'{} was (not modified)'.format(mod_filename), out, MULTILINE),
            'result': False},
        3: {'args': (r'Output (saved) in: "1/{}"{}Original unaltered: "{}"'.format(mod_filename, linesep, mod_filename), out, MULTILINE),
            'result': True},
        4: {'args': (r'Can\'t locate Config/IniFiles.pm in @INC \(you may need to install the (Config::IniFiles module)\)', err, MULTILINE),
            'result': False},
        5: {'args': (r'(Usage): tes3cmd COMMAND OPTIONS plugin...', err, MULTILINE),
            'result': True},
    }
    for data in ceases.values():
        match = search(*data['args'])  # type: ignore
        if match:
            return data['result'], match.group(1)  # type: ignore
